use coremem::{self, Driver, GenericSim};
use coremem::sim::legacy::SimState;
use coremem::sim::spirv::SpirvSim;
use coremem::geom::Index;
use std::time::{Instant, Duration};

fn measure<F: FnMut()>(name: &str, n_times: u32, mut f: F) -> f32 {
    let mut times = vec![];
    for _ in 0..n_times {
        let start = Instant::now();
        f();
        let end = Instant::now();
        times.push(end - start);
    }
    let avg = (times.into_iter().sum::<Duration>() / n_times).as_secs_f32();
    println!("[{}]: {}s", name, avg);
    avg
}

fn measure_steps<S: GenericSim + Clone + Default + Send + Sync + 'static>(name: &str, steps_per_call: u32, mut d: Driver<S>) {
    measure(name, 100/steps_per_call, || d.step_multiple(steps_per_call));
}

fn main() {
    coremem::init_logging();
    measure_steps("spirv/80", 1, Driver::<SpirvSim>::new_spirv(Index::new(80, 80, 80), 1e-3));
    measure_steps("sim/80", 1, Driver::<SimState>::new(Index::new(80, 80, 80), 1e-3));
    measure_steps("spirv/80 step(2)", 2, Driver::<SpirvSim>::new_spirv(Index::new(80, 80, 80), 1e-3));
    measure_steps("sim/80 step(2)", 2, Driver::<SimState>::new(Index::new(80, 80, 80), 1e-3));
    measure_steps("spirv/80 step(10)", 10, Driver::<SpirvSim>::new_spirv(Index::new(80, 80, 80), 1e-3));
    measure_steps("sim/80 step(10)", 10, Driver::<SimState>::new(Index::new(80, 80, 80), 1e-3));
    measure_steps("spirv/80 step(100)", 100, Driver::<SpirvSim>::new_spirv(Index::new(80, 80, 80), 1e-3));
    measure_steps("sim/80 step(100)", 100, Driver::<SimState>::new(Index::new(80, 80, 80), 1e-3));
}

