//! database of common materials

use super::{AnisomorphicConductor, IsomorphicConductor, LinearMagnet, Ferroxcube3R1, MinimalSquare};
use crate::real::Real;
use crate::types::vec::Vec3;

pub fn conductor<R: Real, R2: Real>(conductivity: R2) -> IsomorphicConductor<R> {
    IsomorphicConductor::new(conductivity.cast())
}
pub fn anisotropic_conductor<R>(conductivity: Vec3<R>) -> AnisomorphicConductor<R> {
    AnisomorphicConductor::new(conductivity)
}

pub fn copper<R: Real>() -> IsomorphicConductor<R> {
    conductor(50_000_000.0)
}

// See https://en.wikipedia.org/wiki/Permeability_(electromagnetism)#Values_for_some_common_materials
/// This is a simplified form of iron annealed in H.
pub fn linear_annealed_iron<R: Real>() -> LinearMagnet<R> {
    LinearMagnet::new(200_000.0)
}
/// This is a simplified form of iron
pub fn linear_iron<R: Real>() -> LinearMagnet<R> {
    LinearMagnet::new(5000.0)
}

/// https://www.ferroxcube.com/upload/media/product/file/MDS/3r1.pdf
pub fn ferroxcube_3r1<R: Real>() -> Ferroxcube3R1<R> {
    Ferroxcube3R1::default()
}
pub fn minimal_square_ferrite<R: Real>() -> MinimalSquare<R> {
    MinimalSquare::default()
}

